import java.util.Date;

import javax.persistence.AssociationOverride;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.Id;
import javax.persistence.Inheritance;
import javax.persistence.InheritanceType;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.MappedSuperclass;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;

@Entity
@Inheritance(strategy = InheritanceType.TABLE_PER_CLASS)
public abstract class Employee {
  @Id
  private int id;

  private String name;

  @Temporal(TemporalType.DATE)
  private Date startDate;

  public int getId() {
    return id;
  }

  public void setId(int id) {
    this.id = id;
  }

  public String getName() {
    return name;
  }

  public void setName(String name) {
    this.name = name;
  }

  public Date getStartDate() {
    return startDate;
  }

  public void setStartDate(Date startDate) {
    this.startDate = startDate;
  }

  public String toString() {
    return "Employee id: " + getId() + " name: " + getName();
  }
}

@MappedSuperclass
abstract class CompanyEmployee extends Employee {
  private int vacation;

  @ManyToOne
  private Employee manager;

  public int getVacation() {
    return vacation;
  }

  public void setVacation(int vacation) {
    this.vacation = vacation;
  }

  public Employee getManager() {
    return manager;
  }

  public void setManager(Employee manager) {
    this.manager = manager;
  }
}

@Entity
@AssociationOverride(name = "manager", joinColumns = @JoinColumn(name = "MGR"))
class PartTimeEmployee extends CompanyEmployee {

  private float hourlyRate;

  public float getHourlyRate() {
    return hourlyRate;
  }

  public void setHourlyRate(float hourlyRate) {
    this.hourlyRate = hourlyRate;
  }

  public String toString() {
    return "PartTimeEmployee id: " + getId() + " name: " + getName();
  }
}

@Entity
class FullTimeEmployee extends CompanyEmployee {
  private long salary;


  private long pensionContribution;

  public long getPensionContribution() {
    return pensionContribution;
  }

  public void setPensionContribution(long pension) {
    this.pensionContribution = pension;
  }

  public long getSalary() {
    return salary;
  }

  public void setSalary(long salary) {
    this.salary = salary;
  }

  public String toString() {
    return "FullTimeEmployee id: " + getId() + " name: " + getName();
  }
}
